from PyQt5.QtCore import QObject, pyqtSlot, QTimer
from time import  strftime
from game.gravitator import Gravitator
from game.brutalforcer import BrutalForcer
from game.triosremover import TriosRemover
from game.movechecker import MoveChecker
from PyQt5.QtGui import QPainter, QBrush, QColor, QFont, QPen
from PyQt5.QtGui import QMouseEvent
from PyQt5.QtWidgets import QWidget
from game.jewelsgenerator import JewelsGenerator

class GameWindow(QWidget):
    def __init__(self):
        super().__init__()

        self.resize(800, 800)
        self.__grid = JewelsGenerator(8, 8, (
            "red",
            "green",
            "blue",
            "white",
            "yellow",
            "purple"
        )).generate_jewels()

        self.__bf = BrutalForcer(self.__grid)
        self.__timer = None
        self.__score = 0
        self.__gravitator = Gravitator(self.__grid)
        self.__remover = TriosRemover()
        self.__move_checker = MoveChecker(self.__grid)
        self.__duo = [None, None]


    @pyqtSlot(int)
    def start_with_time(self, seconds):
        self.__updater = QTimer()
        self.__updater.setInterval(1000)
        self.__updater.setSingleShot(False)
        self.__updater.timeout.connect(self.update)

        self.__timer = QTimer()
        self.__timer.setInterval(seconds * 1000)
        self.__timer.setSingleShot(True)
        self.__timer.timeout.connect(self.game_over)

        self.__timer.start()
        self.__updater.start()
        self.show()

    def game_over(self):
        self.close()
        print("game over")

    def mouseReleaseEvent(self, e):
        """
        :type e: QMouseEvent
        """
        #self.__bf.do_it()
        #return

        x = int(e.x() / 50)
        y = int(e.y() / 50)

        if x > 7 or y > 7:
            self.__duo = [None, None]
            self.update()
            return

        if self.__duo[0]:
            self.__duo[1] = [x, y]
        else:
            self.__duo[0] = [x, y]

        self.update()

        if self.__duo[0] and self.__duo[1]:
            if self.__move_checker.check_move(
                self.__duo[0][0],
                self.__duo[0][1],
                self.__duo[1][0],
                self.__duo[1][1]
            ):

                number_of_removed = self.__remover.remove_trios(self.__grid)
                while number_of_removed:
                    self.__score += number_of_removed * 3
                    self.__gravitator.apply_gravity()
                    self.update()
                    number_of_removed = self.__remover.remove_trios(self.__grid)
            else:
                print("Nevalidní krok")

            self.__duo = [None, None]

    def paintEvent(self, paint_event):
        p = QPainter(self)


        """for r in self.__grid.values():
            print(len(r.values()))"""

        for y, row in self.__grid.items():
            s_y = len(self.__grid.keys()) - y

            for x, value in row.items():
                if self.__grid[y][x]:
                    p.setBrush(QBrush(QColor(self.__grid[y][x])))
                    p.drawRect(x * 50, y * 50, 50, 50)

            p.setBrush(QBrush(QColor("transparent")))
            p.setPen(QPen(QColor("orange"), 5))

            if self.__duo[0]:
                p.drawRect(self.__duo[0][0] * 50, self.__duo[0][1] * 50, 50, 50)
            if self.__duo[1]:
                p.drawRect(self.__duo[1][0] * 50, self.__duo[1][1] * 50, 50, 50)


            p.setBrush(QBrush(QColor("black")))
            p.setPen(QPen(QColor("black")))
            p.setFont(QFont("Arial", 40))
            p.drawText(420, 100, str(self.__score))

            if self.__timer:
                seconds = int(self.__timer.remainingTime() / 1000)
                p.drawText(420, 50, "{}:{}".format(int(seconds / 60), seconds % 60))
